Drawer.TOP = 0;
Drawer.RIGHT = 1;
Drawer.BOTTOM = 2;
Drawer.LEFT = 3;
Drawer.all = new Object();

function Drawer(id, speed, side, open, onstart, onmove, onend) {
    var outer = document.getElementById(id);

    this.id = id;
    this.speed = speed;
    this.side = side;
    this.open = open || false;
    this.onstart = onstart;
    this.onmove = onmove;
    this.onend = onend;

    this.moving = false;
    this.outer = outer;
    this.inner = outer.getElementsByTagName("div").item(0);
    this.by = 0;

    this.inner.style.position = "relative";
    this.outer.style.visibility = this.open ? "visible" : "hidden";
    Drawer.all[id] = this;
}

Drawer.prototype = {

    update: function() {
        this.width = this.outer.clientWidth || this.outer.offsetWidth || 0;
        this.height = this.outer.clietHeight || this.outer.offsetHeight || 0;

        switch (this.side) {
        case Drawer.TOP:
            this.clip = [(this.open ? 0 : this.height), this.width, this.height, 0];
            this.style = "top";
            this.start = 0;
            break;
        case Drawer.RIGHT:
            this.clip = [0, (this.open ? this.width : 0), this.height, 0];
            this.style = "left";
            this.start = -this.width;
            break;
        case Drawer.BOTTOM:
            this.clip = [0, this.width, (this.open ? this.height : 0), 0];
            this.style = "top";
            this.start = -this.height;
            break;
        case Drawer.LEFT:
            this.clip = [0, this.width, this.height, (this.open ? 0 : this.width)];
            this.style = "left";
            this.start = 0;
            break;
        }

        this.outer.style.visibility = "visible";
    },

    activate: function() {
        if (typeof this.onstart == "function") this.onstart();
        this.update();
        this.move();
        this.moving = true;
    },

    move: function() {
        var end;
        var dir;
        var i = this.side;

        switch (i) {
        case Drawer.TOP:
            end = this.open ? this.height : 0;
            dir = -1;
            break;
        case Drawer.RIGHT:
            end = this.open ? 0 : this.width;
            dir = 1;
            break;
        case Drawer.BOTTOM:
            end = this.open ? 0 : this.height;
            dir = 1;
            break;
        case Drawer.LEFT:
            end = this.open ? this.width : 0;
            dir = -1;
            break;
        }

        this.by = Math.round((end - this.clip[i]) * this.speed) || 1;
        this.clip[i] += ((this.open ? -1 : 1) * Math.abs(this.by)) * dir;
        this.outer.style.clip = "rect(" + this.clip.join("px ") + "px)";
        this.inner.style[this.style] = (this.start + this.clip[i]) + "px";

        if (typeof this.onmove == "function") this.onmove();

        if (this.clip[i] == end) {
            this.open = !this.open;
            this.moving = false;
            if (this.open == false) this.outer.style.visibility = "hidden";
            else this.outer.style.clip = "auto";
            if (typeof this.onend == "function") this.onend();
            return;
        }

        setTimeout("Drawer.all['"+this.id+"'].move()", 10);
    }
}
